#!/usr/bin/env python
"""
Technical Indicators for Financial Trading

Usage:
    tifft -h|--help
    tifft --version
    tifft history [--debug|--info] [--max-rows=<int>] [--start=<date>]
        [--end=<date>] [--csv=<path>] <symbol>...
    tifft macd [--debug|--info] [--max-rows=<int>] [--start=<date>]
        [--end=<date>] [--fast-ema-span=<int>] [--slow-ema-span=<int>]
        [--macd-ema-span=<int>] [--csv=<path>] <symbol>

Options:
    -h, --help              Print help and exit
    --version               Print version and exit
    --debug, --info         Execute a command with debug|info messages
    --max-rows=<int>        Specify the max rows to display [default: 60]
    --start=<date>          Specify the start date (e.g., 2021-01-01)
    --end=<date>            Specify the end date
    --csv=<path>            Write data with CSV into a file
    --fast-ema-span=<int>   Specify the fast EMA span [default: 12]
    --slow-ema-span=<int>   Specify the slow EMA span [default: 26]
    --macd-ema-span=<int>   Specify the MACD EMA span [default: 9]

Commands:
    history                 Fetch historical data from FRED
    macd                    Fetch historical data from FRED and calculate MACD

Arguments:
    <symbol>                Data symbol
"""

import logging
import os

from docopt import docopt

from . import __version__
from .datareader import calculate_macd_from_fred_data, fetch_fred_data


def main():
    args = docopt(__doc__, version=__version__)
    _set_log_config(debug=args['--debug'], info=args['--info'])
    logger = logging.getLogger(__name__)
    logger.debug(f'args:{os.linesep}{args}')
    if args['history']:
        fetch_fred_data(
            symbols=args['<symbol>'], output_csv_path=args['--csv'],
            start_date=args['--start'], end_date=args['--end'],
            max_rows=args['--max-rows']
        )
    elif args['macd']:
        calculate_macd_from_fred_data(
            symbol=args['<symbol>'][0], output_csv_path=args['--csv'],
            start_date=args['--start'], end_date=args['--end'],
            max_rows=args['--max-rows'],
            fast_ema_span=int(args['--fast-ema-span']),
            slow_ema_span=int(args['--slow-ema-span']),
            macd_ema_span=int(args['--macd-ema-span'])
        )


def _set_log_config(debug=None, info=None):
    if debug:
        lv = logging.DEBUG
    elif info:
        lv = logging.INFO
    else:
        lv = logging.WARNING
    logging.basicConfig(
        format='%(asctime)s %(levelname)-8s %(message)s',
        datefmt='%Y-%m-%d %H:%M:%S', level=lv
    )



#!/usr/bin/env python

import logging
import os
from pprint import pformat

import pandas as pd


class MacdCalculator(object):
    def __init__(self, fast_ema_span=12, slow_ema_span=26, macd_ema_span=9,
                 **kwargs):
        assert fast_ema_span < slow_ema_span, 'invalid spans'
        self.__logger = logging.getLogger(__name__)
        self.__fast_ema_span = fast_ema_span
        self.__slow_ema_span = slow_ema_span
        self.__macd_ema_span = macd_ema_span
        self.__ewm_kwargs = {'adjust': False, **kwargs}
        self.__logger.debug(f'vars(self):{os.linesep}' + pformat(vars(self)))

    def calculate_oscillator(self, values):
        return (
            values.to_frame(name='value') if isinstance(values, pd.Series)
            else pd.DataFrame({'value': values})
        ).assign(
            macd=lambda d: (
                d['value'].ewm(
                    span=self.__fast_ema_span, **self.__ewm_kwargs
                ).mean() - d['value'].ewm(
                    span=self.__slow_ema_span, **self.__ewm_kwargs
                ).mean()
            )
        ).assign(
            macd_ema=lambda d: d['macd'].ewm(
                span=self.__macd_ema_span, **self.__ewm_kwargs
            ).mean()
        ).assign(
            macd_ema_diff=lambda d: (d['macd'] - d['macd_ema'])
        ).assign(
            signal=lambda d: d['macd_ema_diff'].mask(
                (d['macd_ema_diff'] > 0) & (d['macd'] > 0), 2
            ).mask(
                d['macd_ema_diff'] > 0, 1
            ).mask(
                (d['macd_ema_diff'] < 0) & (d['macd'] < 0), -2
            ).mask(
                d['macd_ema_diff'] < 0, -1
            ).astype(int)
        ).drop(columns='macd_ema_diff')



#!/usr/bin/env python

import logging
import os
from pathlib import Path

import pandas as pd
import pandas_datareader.data as pdd

from .macd import MacdCalculator


def fetch_fred_data(symbols, output_csv_path=None, start_date=None,
                    end_date=None, max_rows=None, **kwargs):
    logger = logging.getLogger(__name__)
    pd.set_option('display.max_rows', (int(max_rows) if max_rows else None))
    print('>>\tGet historical data from FRED:\t' + ', '.join(symbols))
    df_hist = _fetch_fred_df(
        symbols=symbols, start_date=start_date, end_date=end_date,
        max_rows=max_rows, **kwargs
    )
    logger.debug(f'df_hist:{os.linesep}{df_hist}')
    print(df_hist)
    if output_csv_path:
        _write_df_to_csv(df=df_hist, csv_path=output_csv_path)


def _fetch_fred_df(symbols, start_date=None, end_date=None, max_rows=None,
                   **kwargs):
    return pdd.DataReader(
        name=symbols, data_source='fred', start=start_date, end=end_date,
        **kwargs
    )


def _write_df_to_csv(df, csv_path):
    output_csv = Path(csv_path).resolve()
    print(f'>>\tWrite a CSV file:\t{output_csv}')
    df.to_csv(output_csv, mode='w', header=True, sep=',')


def calculate_macd_from_fred_data(symbol, output_csv_path=None,
                                  start_date=None, end_date=None,
                                  max_rows=None, fast_ema_span=12,
                                  slow_ema_span=26, macd_ema_span=9, **kwargs):
    logger = logging.getLogger(__name__)
    pd.set_option('display.max_rows', (int(max_rows) if max_rows else None))
    print(f'>>\tGet historical data from FRED:\t{symbol}')
    df_hist = _fetch_fred_df(
        symbols=symbol, start_date=start_date, end_date=end_date,
        max_rows=max_rows
    )
    logger.debug(f'df_hist:{os.linesep}{df_hist}')
    print(f'>>\tCalculate MACD:\t{symbol}')
    macdc = MacdCalculator(
        fast_ema_span=fast_ema_span, slow_ema_span=slow_ema_span,
        macd_ema_span=macd_ema_span, **kwargs
    )
    df_macd = macdc.calculate_oscillator(values=df_hist.iloc[:, 0]).rename(
        columns={
            'value': symbol, 'macd': 'MACD', 'macd_ema': 'MACD_EMA',
            'signal': 'SIGNAL'
        }
    )
    logger.debug(f'df_macd:{os.linesep}{df_macd}')
    print(df_macd)
    if output_csv_path:
        _write_df_to_csv(df=df_macd, csv_path=output_csv_path)



#!/usr/bin/env python

from setuptools import find_packages, setup

from tifft import __version__

with open('README.md', 'r') as f:
    long_description = f.read()

setup(
    name='tifft',
    version=__version__,
    author='Daichi Narushima',
    author_email='dnarsil+github@gmail.com',
    description='Technical Indicators for Financial Trading',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/dceoy/tifft',
    packages=find_packages(),
    include_package_data=True,
    install_requires=['docopt', 'pandas', 'pandas-datareader'],
    entry_points={'console_scripts': ['tifft=tifft.cli:main']},
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Console',
        'Intended Audience :: Financial and Insurance Industry',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Operating System :: POSIX :: Linux',
        'Operating System :: MacOS',
        'Programming Language :: Python :: 3',
        'Topic :: Office/Business :: Financial :: Investment'
    ],
    python_requires='>=3.6'
)
